<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\Furmugations;
use Modules\Sd\Entities\FurmugationsAttachment;
use Modules\Sd\Entities\Shipment;
use Modules\Sd\Entities\ShippingAgent;

class FurmugationsController extends Controller
{
    public function getAllFurmugations()
    {
        try {
            $query = 'SELECT * FROM sd_furmugations ORDER BY `created_at` DESC';
            $furmugations_data = DB::select($query);
            $result = array();

            foreach ($furmugations_data as $data) {
                array_push($result, [
                    "customer_order_id" => $data->customer_order_id,
                    "factory_po" => $data->factory_po,
                    "shipment_id" => $data->shipment_id,
                    "furmugations_id" => $data->furmugations_id,
                ]);
            }
            $responseBody = $this->responseBody(true, "FurmugationsController", "getAllFurmugations", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "getAllFurmugations", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function saveData(Request $request)
    {
        try {
            $furmugations_data = new Furmugations();
            $furmugations_data->customer_order_id = $request->get('customer_order_id');
            $furmugations_data->factory_po = $request->get('factory_po');
            $furmugations_data->shipment_id = $request->get('shipment_id');
            $furmugations_data->note = $request->get('note');

            if ($furmugations_data->save()) {
                // $this->sendEmail();
                $attachments = FurmugationsAttachment::where([['furmugations_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->furmugations_id = $furmugations_data->furmugations_id;
                    $attachment->update();
                }
                $attachments = FurmugationsAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->furmugations_id = $furmugations_data->furmugations_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "FurmugationsController", "saveData", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "saveData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function update(Request $request, $id)
    {
        try {
            $furmugation_data = Furmugations::find($id);
            $furmugation_data->customer_order_id = $request->get('customer_order_id');
            $furmugation_data->factory_po = $request->get('factory_po');
            $furmugation_data->shipment_id = $request->get('shipment_id');
            $furmugation_data->note = $request->get('note');
            if ($furmugation_data->update()) {
                $attachments = FurmugationsAttachment::where([['furmugations_attachment_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->furmugations_id = $furmugation_data->furmugations_id;
                    $attachment->update();
                }
                $attachments = FurmugationsAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->furmugations_id = $furmugation_data->furmugations_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "FurmugationsController", "update", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "update", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function delete(Request $request, $id)
    {
        try {
            $furmugations_data = Furmugations::where('furmugations_id', '=', $id);
            $rec_data_attachments = FurmugationsAttachment::where('furmugations_id', '=', $id)->get();

            if ($furmugations_data->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "FurmugationsController", "delete", "success");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "delete", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getFurmugationsData($id)
    {
        try {
            $recordHead = Furmugations::find($id);

            $responseBody = $this->responseBody(true, "FurmugationsController", "getFurmugationsData", $recordHead);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShipmentData()
    {
        try {
            $shipmet = Shipment::all();

            $responseBody = $this->responseBody(true, "FurmugationsController", "getShipmentData", $shipmet);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "getShipmentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getShippingAgentData()
    {
        try {
            $shipping_agent = ShippingAgent::all();

            $responseBody = $this->responseBody(true, "FurmugationsController", "getShippingAgentData", $shipping_agent);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "getShippingAgentData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            sd_shipment.customer_order_id,
            sd_shipment.shipment_id,
            pp_customer_orders.factory_po_num
        FROM
            sd_shipment
            LEFT JOIN pp_customer_orders ON sd_shipment.customer_order_id = pp_customer_orders.order_id
        WHERE
            sd_shipment.shipment_id = ' . $id;
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_id" => $order->customer_order_id,
                    "factory_po_num" => $order->factory_po_num,
                    "shipment_id" => $order->shipment_id
                ]);
            }


            $responseBody = $this->responseBody(true, "FurmugationsController", "getCustomerOrderData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "FurmugationsController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = 'http://127.0.0.1:8000/furmugations/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('furmugations'), $filename);

        $furmugations_data = new FurmugationsAttachment();
        $furmugations_data->furmugations_id = 0;
        $furmugations_data->description = $description;
        $furmugations_data->file_path = $filename;
        $furmugations_data->token = $token;
        return $furmugations_data->save();
    }

    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_furmugations_attachment WHERE  furmugations_id = "' . $id . '" OR furmugations_id = "0"');
            $responseBody = $this->responseBody(true, "furmugationsController", "allScheduleAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "furmugationsController", "allScheduleAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = FurmugationsAttachment::where('furmugations_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "furmugationsController", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "furmugationsController", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
