<?php

namespace Modules\Sd\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\VesselAvailability;
use Modules\Sd\Entities\VesselAvailabilityAttachment;
use Modules\St\Entities\Currency;

class VesselAvailabilityController extends Controller
{
    public function getAvailabilityHeader($id)
    {
        try {
            $query = 'SELECT pp_customer_orders.factory_po_num,
            st_customer_deliveries.address,
            pp_customer_orders.purchase_order,
            pp_customer_orders.delivery_date
            FROM pp_customer_orders
            LEFT JOIN st_customer_deliveries ON pp_customer_orders.customer_id = st_customer_deliveries.customer_id
            WHERE pp_customer_orders.order_id = "' . $id . '"';
            $result = DB::select($query)[0];
            if ($result) {
                $result->currency = Currency::all();
            }

            $responseBody = $this->responseBody(true, "VesselAvailabilityController", "getAvailabilityHeader", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselAvailabilityController", "getAvailabilityHeader", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function uploadAvailabilityAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = 'http://127.0.0.1:8000/order/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('order'), $filename);

        $vessel_attachment = new VesselAvailabilityAttachment();
        $vessel_attachment->vessel_availability_attachment_id = 0;
        $vessel_attachment->description = $description;
        $vessel_attachment->file_path = $filename;
        $vessel_attachment->token = $token;
        return $vessel_attachment->save();
    }



    public function allAvailabilityAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT *FROM sd_vessel_availability_attachments WHERE  vessel_availability_id = "' . $id . '" OR vessel_availability_id = "0"');
            $responseBody = $this->responseBody(true, "VesselAvailabilityController", "allVesselAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselAvailabilityController", "allVesselAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function saveAvailability(Request $request)
    {
        try {
            VesselAvailability::where('order_id', '=', $request->get('order_id'))->delete();
            $vesselAvailability = new VesselAvailability();
            $vesselAvailability->factory_order_no = $request->get('factory_order_no');
            $vesselAvailability->eta = $request->get('eta');
            $vesselAvailability->customer_po = $request->get('customer_po');
            $vesselAvailability->designation = $request->get('designation');
            $vesselAvailability->cargo_ready_date = $request->get('cargo_ready_date');
            $vesselAvailability->suggest_vessel = $request->get('suggest_vessel');
            $vesselAvailability->suggest_date = $request->get('suggest_date');
            $vesselAvailability->shipping_rate = $request->get('shipping_rate');
            $vesselAvailability->currency_id = $request->get('currency_id');
            $vesselAvailability->order_id = $request->get('order_id');
            if ($vesselAvailability->save()) {
                $attachments = VesselAvailabilityAttachment::where([['vessel_availability_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->vessel_availability_id = $vesselAvailability->vessel_availability_id;
                    $attachment->update();
                }
                $attachments = VesselAvailabilityAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->vessel_availability_id = $vesselAvailability->vessel_availability_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "VesselAvailabilityController", "saveAvailability", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselAvailabilityController", "saveAvailability", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function load_saved_availability($id)
    {
        try {
            $availability = VesselAvailability::where('order_id', '=', $id)->first();
            $availability->current_availability = false;
            if ($availability) {
                $availability->attachment = VesselAvailabilityAttachment::where('vessel_availability_id', '=', $availability->vessel_availability_id)->get();
                $availability->current_availability = true;
            }
            $responseBody = $this->responseBody(true, "VesselAvailabilityController", "load_saved_availability", $availability);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselAvailabilityController", "load_saved_availability", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
