<?php

namespace Modules\Sd\Http\Controllers;

use App\Http\Controllers\MailController;
use App\Mail\MailData;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Sd\Entities\VesselAvailability;
use Modules\Sd\Entities\VesselAvailabilityAttachment;
use Modules\Sd\Entities\VesselSchedule;
use Modules\Sd\Entities\VesselScheduleAttachment;
use Modules\St\Entities\Currency;
use Exception;

class VesselScheduledController extends Controller
{
    public function getScheduleData()
    {
        try {
            $query = 'SELECT * FROM sd_vessel_schedules ORDER BY `created_at` DESC';
            $schedules = DB::select($query);
            $result = array();

            foreach ($schedules as $schedule) {
                array_push($result, [
                    "vessel_schedule_id" => $schedule->vessel_schedule_id,
                    "external_id" => $schedule->external_id,
                    "factory_order_no" => $schedule->factory_order_no,
                    "eta" => $schedule->eta,
                    "customer_po" => $schedule->customer_po,
                    "designation" => $schedule->designation,
                    "cargo_ready_date" => $schedule->cargo_ready_date,
                    "suggest_vessel" => $schedule->suggest_vessel,
                    "tentative_arrival_date" => $schedule->tentative_arrival_date,
                    "shipping_rate" => $schedule->shipping_rate,
                    "seen" => $schedule->seen,
                    "status" => $schedule->status,
                ]);
            }
            $responseBody = $this->responseBody(true, "VesselScheduledController", "getScheduleData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "getScheduleData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getVsData($id)
    {
        try {
            $recordHead = VesselSchedule::find($id);
            $recordData = VesselSchedule::where('vessel_schedule_id', '=', $id)->get();

            foreach ($recordData as $data) {
                $total = $data->seal_damage + $data->block_damage + $data->slab_damage + $data->print_damage + $data->packing_damage + $data->other_damage;
                $qc_total = $data->qc_seal_damage + $data->qc_block_damage + $data->qc_slab_damage + $data->qc_print_damage + $data->qc_packing_damage + $data->qc_other_damage;
                $data->total = $total;
                $data->qc_total = $qc_total;
            }

            $responseBody = $this->responseBody(true, "VesselScheduledController", "getVcData", $recordHead);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCurrencyPrimaryID()
    {
        try {
            $query = 'SELECT currency_id,iso_code  FROM st_currencies WHERE `status` = 1';
            $currencies = DB::select($query);
            $result = array();

            foreach ($currencies as $currency) {
                array_push($result, [
                    "currency_id" => $currency->currency_id,
                    "currency_code" => $currency->iso_code,
                ]);
            }

            $responseBody = $this->responseBody(true, "VesselScheduledController", "getCurrencyData", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "getCurrencyData", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }
    public function getScheduleHeader($id)
    {
        try {
            $query = 'SELECT pp_customer_orders.factory_po_num,
            st_customer_deliveries.address,
            pp_customer_orders.purchase_order,
            pp_customer_orders.delivery_date
            FROM pp_customer_orders
            LEFT JOIN st_customer_deliveries ON pp_customer_orders.customer_id = st_customer_deliveries.customer_id
            WHERE pp_customer_orders.order_id = "' . $id . '"';
            $result = DB::select($query)[0];
            if ($result) {
                $result->currency = Currency::all();
            }

            $responseBody = $this->responseBody(true, "VesselScheduledController", "getScheduleHeader", $result);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "getScheduleHeader", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function uploadScheduleAttachment(Request $request)
    {

        $description = $request->get('foo');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = 'http://127.0.0.1:8000/vessel_schedule_attachment/' . uniqid() . '_' . time() . '.' . str_replace(' ', '_', $file_name);
        //$filename = 'https://erp.rbs.lk/order/' . uniqid() . '_' . time() . '.' .str_replace(' ', '_', $file_name);
        $file->move(public_path('vessel_schedule_attachment'), $filename);

        $vessel_attachment = new VesselScheduleAttachment();
        $vessel_attachment->vessel_schedule_attachment_id = 0;
        $vessel_attachment->description = $description;
        $vessel_attachment->file_path = $filename;
        $vessel_attachment->token = $token;
        return $vessel_attachment->save();
    }



    public function allScheduleAttachment($id)
    {
        try {

            $attachment = DB::select('SELECT * FROM sd_vessel_schedule_attachments WHERE  vessel_schedule_id = "' . $id . '" OR vessel_schedule_id = "0"');
            $responseBody = $this->responseBody(true, "VesselScheduledController", "allVesselAttachment", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "allVesselAttachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function saveSchedule(Request $request)
    {
        try {
            $vesselSchedule = new VesselSchedule();
            $vesselSchedule->factory_order_no = $request->get('factory_order_no');
            $vesselSchedule->external_id = $request->get('external_id');
            $vesselSchedule->eta = $request->get('eta');
            $vesselSchedule->customer_po = $request->get('customer_po');
            $vesselSchedule->designation = $request->get('designation');
            $vesselSchedule->cargo_ready_date = $request->get('cargo_ready_date');
            $vesselSchedule->suggest_vessel = $request->get('suggest_vessel');
            $vesselSchedule->tentative_arrival_date = $request->get('tentative_arrival_date');
            $vesselSchedule->shipping_rate = $request->get('shipping_rate');
            $vesselSchedule->currency_id = $request->get('currency_id');
            $vesselSchedule->note = $request->get('note');
            $vesselSchedule->order_id = $request->get('order_id');
            if ($vesselSchedule->save()) {
                // $this->sendEmail();
                $attachments = VesselScheduleAttachment::where([['vessel_schedule_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->vessel_schedule_id = $vesselSchedule->vessel_schedule_id;
                    $attachment->update();
                }
                $attachments = VesselScheduleAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->vessel_schedule_id = $vesselSchedule->vessel_schedule_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "VesselScheduledController", "saveSchedule", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "saveSchedule", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function deleteSchedule(Request $request, $id)
    {
        try {
            $vesselSchedule = VesselSchedule::where('vessel_schedule_id', '=', $id);
            $rec_data_attachments = VesselScheduleAttachment::where('vessel_schedule_id', '=', $id)->get();

            if ($vesselSchedule->delete()) {
                foreach ($rec_data_attachments as $rec_data_attachment) {
                    $rec_data_attachment->delete();
                }
            }

            $responseBody = $this->responseBody(true, "VesselScheduledController", "deleteSchedule", "sucess");
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "deleteSchedule", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function updateSchedule(Request $request, $id)
    {
        try {
            $vesselSchedule = VesselSchedule::find($id);
            $vesselSchedule->factory_order_no = $request->get('factory_order_no');
            $vesselSchedule->external_id = $request->get('external_id');
            $vesselSchedule->eta = $request->get('eta');
            $vesselSchedule->customer_po = $request->get('customer_po');
            $vesselSchedule->designation = $request->get('designation');
            $vesselSchedule->cargo_ready_date = $request->get('cargo_ready_date');
            $vesselSchedule->suggest_vessel = $request->get('suggest_vessel');
            $vesselSchedule->tentative_arrival_date = $request->get('tentative_arrival_date');
            $vesselSchedule->shipping_rate = $request->get('shipping_rate');
            $vesselSchedule->currency_id = $request->get('currency_id');
            $vesselSchedule->note = $request->get('note');
            $vesselSchedule->order_id = $request->get('order_id');
            if ($vesselSchedule->update()) {
                $attachments = VesselScheduleAttachment::where([['vessel_schedule_id', '=', 0], ['token', '=', $request->token]])->get();
                foreach ($attachments as $attachment) {
                    $attachment->vessel_schedule_id = $vesselSchedule->vessel_schedule_id;
                    $attachment->update();
                }
                $attachments = VesselScheduleAttachment::where('token', '=', $request->token)->get();
                foreach ($attachments as $attachment) {
                    $attachment->vessel_schedule_id = $vesselSchedule->vessel_schedule_id;
                    $attachment->update();
                }
            }

            $responseBody = $this->responseBody(true, "VesselScheduledController", "updateSchedule", $request->token);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "updateSchedule", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function load_saved_schedule($id)
    {
        try {
            $schedule = VesselSchedule::where('order_id', '=', $id)->first();
            $schedule->current_schedule = false;
            if ($schedule) {
                $schedule->attachment = VesselScheduleAttachment::where('vessel_schedule_id', '=', $schedule->vessel_schedule_id)->get();
                $schedule->current_schedule = true;
            }
            $responseBody = $this->responseBody(true, "VesselScheduledController", "load_saved_schedule", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "load_saved_schedule", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function deleteScheduleAttachment($id)
    {
        try {
            $rec_data = VesselScheduleAttachment::where('vessel_schedule_attachment_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $schedule = $data;
                $data->delete();
            }
            $schedule = true;
            $responseBody = $this->responseBody(true, "VesselScheduleAttachment", "delete schedule attachment", $schedule);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduleAttachment", "delete schedule attachment", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getFactoryOrderNo()
    {
        try {
            $query = "SELECT
            *
        FROM
            pp_customer_orders
        WHERE
            production_status > '0'";
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "factory_po_num" => $order->factory_po_num,
                ]);
            }


            $responseBody = $this->responseBody(true, "VesselScheduledController", "getFactoryOrderNo", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getCustomerOrderData($id)
    {
        try {
            $query = 'SELECT
            pp_customer_orders.factory_po_num,
            st_customer_deliveries.address,
            pp_customer_orders.purchase_order,
            pp_customer_orders.delivery_date,
            pp_customer_orders.order_id
        FROM
            pp_customer_orders
            LEFT JOIN st_customer_deliveries ON pp_customer_orders.delivery_address_id = st_customer_deliveries.id
        WHERE
            pp_customer_orders.factory_po_num = "' . $id . '"';
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "purchase_order" => $order->purchase_order,
                    "delivery_address" => $order->address,
                    "delivery_date" => $order->delivery_date,
                    "order_id" => $order->order_id,
                    "factory_order_no" => $order->factory_po_num
                ]);
            }


            $responseBody = $this->responseBody(true, "VesselScheduledController", "getCustomerOrderData", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "VesselScheduledController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function genarateExternalID()
    {
        $id = 0;
        $result = DB::select('SELECT MAX(external_id) AS id FROM sd_vessel_schedules')[0];
        if ($result) {
            $id = $result->id;
        }
        return response()->json(["external_id" => ($id += 1)]);

    }

    public function sendEmail()
    {
        $subject = "test";
        $email_body = "body";
        $receiver = "unzoysa.un@gmail.com";
        MailController::createMail($subject, $email_body, MailData::getSender(), $receiver);
    }




    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
